package net.ihe.gazelle.adapter.connector;

import org.hl7.fhir.r4.model.Bundle;
import org.hl7.fhir.r4.model.MessageHeader;
import org.hl7.fhir.r4.model.Meta;
import org.hl7.fhir.r4.model.Narrative;

public class DeletionBundle {
    /**
     * private constructor to override the implicit one.
     */
    private DeletionBundle() {
    }

    public static Bundle prepareBundleAfterDeletionOperation(String uuid, String url) {
        Bundle response = new Bundle();
        response.setId("BundlePIXmResponse");

        Meta metadata = new Meta();
        metadata.addProfile("http://fhir.ch/ig/ch-epr-mhealth/StructureDefinition/ch-pixm-bundle-response");
        response.setType(Bundle.BundleType.MESSAGE);
        response.setMeta(metadata);


        Bundle.BundleEntryComponent entry = new Bundle.BundleEntryComponent();
        entry.setFullUrl(url + "/pixm_fhir_server/fhir_ch/Patient/" + uuid);

        MessageHeader messageHeader = new MessageHeader();
        messageHeader.setId(uuid);
        Narrative narrative = new Narrative();
        narrative.setStatus(Narrative.NarrativeStatus.GENERATED);
        messageHeader.setText(narrative);

        MessageHeader.MessageHeaderResponseComponent mhrc = new MessageHeader.MessageHeaderResponseComponent();
        mhrc.setIdentifier(uuid);
        mhrc.setCode(MessageHeader.ResponseType.OK);

        messageHeader.setResponse(mhrc);
        entry.setResource(messageHeader);

        entry.getRequest()
                .setUrl("Delete")
                .setMethod(Bundle.HTTPVerb.DELETE);
        entry.getResponse()
                        .setStatus("DONE");
        response.addEntry(entry);


        return response;

    }
}
