package net.ihe.gazelle.adapter.servlet;

import ca.uhn.fhir.context.FhirContext;
import ca.uhn.fhir.narrative.DefaultThymeleafNarrativeGenerator;
import ca.uhn.fhir.narrative.INarrativeGenerator;
import ca.uhn.fhir.rest.server.IResourceProvider;
import ca.uhn.fhir.rest.server.RestfulServer;
import ca.uhn.fhir.rest.server.exceptions.InternalErrorException;
import ca.uhn.fhir.rest.server.interceptor.ExceptionHandlingInterceptor;
import ca.uhn.fhir.rest.server.interceptor.LoggingInterceptor;
import net.ihe.gazelle.business.provider.CHBundleProvider;
import net.ihe.gazelle.business.provider.ChPatientResourceProvider;

import javax.inject.Inject;
import javax.servlet.annotation.WebServlet;
import java.util.ArrayList;
import java.util.List;

/**
 * This servlet is the actual FHIR server itself
 */
@WebServlet(urlPatterns = {"/fhir_ch/*"}, displayName = "FHIR Server CH")
public class ChHapiFhirServer extends RestfulServer {
    private static final long serialVersionUID = 1L;

    @Inject
    private ChPatientResourceProvider chPatientResourceProvider;

    @Inject
    private CHBundleProvider chBundleProvider;

    /**
     * Constructor
     */
    public ChHapiFhirServer() {
        super(FhirContext.forR4()); // This is an R4 server
    }

    /**
     * This method is called automatically when the
     * servlet is initializing.
     */
    @Override
    public void initialize() {
        /*
         * Two resource providers are defined. Each one handles a specific
         * type of resource.
         */
        List<IResourceProvider> providers = new ArrayList<>();
        providers.add(chPatientResourceProvider);
        providers.add(chBundleProvider);
        setResourceProviders(providers);

        //creating an interceptor to log in console an error occurring
        LoggingInterceptor loggingInterceptor = new LoggingInterceptor();
        loggingInterceptor.setLoggerName("test.accesslog");
        loggingInterceptor.setMessageFormat("Source[$remoteAddr] Operation[${operationType}"
                + "${idOrResourceName}] UA[${requestHeader.user-agent}] Params[${requestParameters}]");

        registerInterceptor(loggingInterceptor);

        //creating an interceptor for special exception to dump the stack trace in the logs.
        ExceptionHandlingInterceptor exceptionInterceptor = new ExceptionHandlingInterceptor();
        exceptionInterceptor.setReturnStackTracesForExceptionTypes(InternalErrorException.class, NullPointerException.class);
        registerInterceptor(exceptionInterceptor);

        /*
         * Use a narrative generator. This is a completely optional step,
         * but can be useful as it causes HAPI to generate narratives for
         * resources which don't otherwise have one.
         */
        INarrativeGenerator narrativeGen = new DefaultThymeleafNarrativeGenerator();
        getFhirContext().setNarrativeGenerator(narrativeGen);


    }

}
