package net.ihe.gazelle.business.provider;

import net.ihe.gazelle.adapter.connector.BusinessToFhirConverter;
import net.ihe.gazelle.adapter.connector.ConversionException;
import net.ihe.gazelle.app.patientregistryapi.application.PatientFeedException;
import net.ihe.gazelle.app.patientregistryapi.business.Patient;
import net.ihe.gazelle.app.patientregistryfeedclient.adapter.PatientFeedClient;
import net.ihe.gazelle.application.PatientRegistryFeedClient;
import net.ihe.gazelle.framework.preferencesmodelapi.application.OperationalPreferencesService;
import net.ihe.gazelle.lib.searchmodelapi.business.exception.SearchException;
import org.hl7.fhir.r4.model.Bundle;

public class PatientRegistryFeedClientMock extends PatientRegistryFeedClient {

	public static final String HTTP_OK = "HTTP_OK";
	public static final String URN_OK = "URN_OK";
	private static final String CROSS_REFERENCE = "Cross Reference";

	public PatientRegistryFeedClientMock() {
	}

	public PatientRegistryFeedClientMock(OperationalPreferencesService operationalPreferencesService) {
		super(operationalPreferencesService);
	}

	public PatientRegistryFeedClientMock(PatientFeedClient searchClient) {
		this.setClient(searchClient);
	}

	@Override
	public Bundle createPatient(net.ihe.gazelle.app.patientregistryapi.business.Patient patient) throws PatientFeedException {

		String uuid = patient.getUuid();
		Bundle bundle = new Bundle();
		bundle.setId("1");

		switch (uuid){
			case "PatientPixMFeed":
				return bundle;
			case "throw_me_something":
				throw new PatientFeedException();
			default:
				return null;
		}
	}

	@Override
	public Bundle updatePatient(net.ihe.gazelle.app.patientregistryapi.business.Patient patientToUpdate, String uuid) throws PatientFeedException {
		if (uuid.equals("42")) {
			throw new PatientFeedException();
		}
		Bundle response = new Bundle();
		Bundle.BundleEntryComponent component = new Bundle.BundleEntryComponent();
		try {
			component.setResource(BusinessToFhirConverter.patientToFhirPatient(patientToUpdate));
		}
		catch (ConversionException e) {
			throw new PatientFeedException(e);
		}
		response.addEntry(component);
		return response;
	}

	@Override
	public Bundle delete(String uuid) throws PatientFeedException{

		if (uuid.equals("42")) {
			throw new PatientFeedException();
		}

		Bundle response = new Bundle();
		response.setId(uuid);

		return response;
	}

}
