package net.ihe.gazelle.business.provider;

import java.util.List;

import net.ihe.gazelle.app.patientregistryxrefsearchclient.adapter.XRefSearchClient;
import org.hl7.fhir.r4.model.Identifier;
import org.hl7.fhir.r4.model.Parameters;
import org.hl7.fhir.r4.model.Parameters.ParametersParameterComponent;

import net.ihe.gazelle.app.patientregistryapi.application.SearchCrossReferenceException;
import net.ihe.gazelle.app.patientregistryapi.business.EntityIdentifier;
import net.ihe.gazelle.application.PatientRegistryXRefSearchClient;
import net.ihe.gazelle.framework.preferencesmodelapi.application.OperationalPreferencesService;

public class PatientRegistryXRefSearchClientMock extends PatientRegistryXRefSearchClient {
	public static final String HTTP_OK = "HTTP_OK";
	public static final String URN_OK = "URN_OK";
	private static final String CROSS_REFERENCE = "Cross Reference";

	public PatientRegistryXRefSearchClientMock() {
	}

	public PatientRegistryXRefSearchClientMock(OperationalPreferencesService operationalPreferencesService) {
		super(operationalPreferencesService);
	}

	public PatientRegistryXRefSearchClientMock(XRefSearchClient xRefSearchClient) {
		setClient(xRefSearchClient);
	}
	@Override
	public Parameters process(EntityIdentifier sourceIdentifier, List<String> targetSystemList) throws SearchCrossReferenceException {
		Parameters response = new Parameters();
		ParametersParameterComponent alias = new ParametersParameterComponent();
		Identifier r4Identifier = new Identifier();
		switch (sourceIdentifier.getSystemIdentifier()){
		case "http://1":
			r4Identifier.setSystem(HTTP_OK);
			r4Identifier.setValue(HTTP_OK);
			alias.setValue(r4Identifier);
			alias.setName(CROSS_REFERENCE);
			response.addParameter(alias);
			return response;
		case "1":
			r4Identifier.setSystem(URN_OK);
			r4Identifier.setValue(URN_OK);
			alias.setValue(r4Identifier);
			alias.setName(CROSS_REFERENCE);
			response.addParameter(alias);
			return response;
		case "0":
			throw new SearchCrossReferenceException(new PatientResourceProviderException("Error in the sourceIdentifier : System does not exit"));
		case "3":
			throw new SearchCrossReferenceException(new PatientResourceProviderException("One of the target domain does not exist"));
		case "4":
			throw new SearchCrossReferenceException(new PatientResourceProviderException("Error in the sourceIdentifier : it does not match " +
		            "any Patient"));
		default:
			return null;
		}
	}
}
